#! /bin/sh

set -x

# This file doubles as a tutorial explaining how to use these tools and
# an actual test of the tools.  Using it as a test can only be done if
# you've already done step 1 and the first part of step 2.
# On a lightly-used 8-proc Pentium III system this whole thing takes a
# couple of hours.
#
# NOTE: These tools don't build a usable kernel, don't boot the kernel,
# and don't run a real test.  They're meant to demonstrate the basic
# functionality of kernel regression hunts and of using BitKeeper for
# those hunts.

# 1.  Set up a local BitKeeper repository.
#
# 2.  Update the reg_env file in this directory and source it.
#
        TOP=`pwd`
        . reg_env
#
# 3.  Set up some files used by the reghunt tools.
#
        bk_setup
#
#     Take a look at the file BK_ChangeSet.linux-2.5 in the builds
#     directory.  The script bk_ids uses that file to convert between
#     various identifiers for a changeset, for example:
#
        bk_ids -f tag -t index v2.6.0-test9
#
# 4.  Run a regression hunt whose config file is available in the
#     hunts/ directory.  The verbosity level for the tools is set to
#     let them write message about what they're doing, and reg_summary
#     can be run at any time to quickly see what's been tested and
#     how many times the build is failing.
#
        cd ${TOP}/hunts
        reg_search bug1.config > bug1.log 2>&1
        reg_summary < bug1.log
#
# 5.  The previous hunt had several build failures.  The changeset IDs
#     for revisions with failing builds are recorded in
#     builds/BK_Failures.linux-2.5.  You can see why each of them failed:
#
        cd ${TOP}/builds/logs.bug*
	grep '^make.* Error' make.*.log
#
#     You'll notice that they all fail for the same reason.  We can
#     identify where the first failure occurs and the patch which fixes
#     it, and add all of the intervening IDs to the list of known build
#     failures.
#
# 6.  Identify the patch which introduced the build failure.  From the
#     output for bug2, we can see that that latest build that we know
#     passed and the earlier that did not.
#
        cd ${TOP}/hunts
        reg_search failstart.config > failstart.log 2>&1
	reg_summary < failstart.log
#
# 7.  Identify the patch which fixed the build failure, based on the.      
#     last failure and first successsful build after it from the output
#     of the bug1 search.
#
        reg_search failend.config > failend.log 2>&1
	reg_summary < failend.log
#
# 8.  Add the intervening range to the list of known failures:
#
        cd ${TOP}/builds
	FIRST=`bk_ids -f bkrev -t index 3f3b2c85d2Fn-c6VDRO0eh6NowjCEw`
	LAST=`bk_ids -f bkrev -t index 3f40ebc0Fpjp9eB1kIIvoZLg8cpLkw`
        reg_addfails ${FIRST} ${LAST} BK_Failures.linux-2.5
#
# 9.  Search for the "cause" of another bug, and notice that it doesn't
#     try to update or build revisions in the range of known build
#     failures.
#
        cd ${TOP}/hunts
        reg_search bug2.config > bug2.log 2>&1
        reg_summary < bug2.log
#
# 10. Search for a bug which was introduced during the range for which
#     the kernel build fails.  We already determined the changeset for
#     which the build starts succeeding.  It's a rather large patch with
#     several unrelated changes, but comments let us guess that the
#     changes to drivers/scsi/qla1280.[ch] are what we want what we want.
#
#     Identify which revisions of those files changed and make a patch:
#
        # cd ${TOP}/builds/linux-2.5/drivers/scsi
	# #cvs log -N qla1280.h | less
	# #cvs log -N qla1280.c | less
	# cd ../..
	# cvs diff -up -r1.17 -r1.18 drivers/scsi/qla1280.h \
	#    > ../qla1280.patch
	# cvs diff -up -r1.35 -r1.36 drivers/scsi/qla1280.c \
	#    >> ../qla1280.patch
#
#     Verify that the patch can be applied at the start of the failing
#     range and allows the build to succeed.
#
	FIRST=`bk_ids -f bkrev -t index 3f3b2c85d2Fn-c6VDRO0eh6NowjCEw`
        bkcvs_update ${FIRST}
	cd ${TOP}/builds/linux-2.5
	patch -p0 < ../qla1280.patch
	kernel_build ${FIRST}
#
#     Perform the hunt using a special-purpose script for REG_UPDATE
#     which applies the patch for changesets within the failure range.
#
        cd ${TOP}/hunts
        reg_search bug3.config > bug3.log 2>&1
        reg_summary < bug3.log


# 2003-12-03  Janis Johnson  <janis187@us.ibm.com>
