#! /bin/sh

set -x

# This file doubles as a tutorial explaining how to use these tools and
# an actual test of the tools.  Using it as a test can only be done if
# you've already done step 1 and the first part of step 2.
# On a lightly-used 8-proc Pentium III system this whole thing takes a
# couple of hours.
#
# NOTE: These tools don't build a usable kernel, don't boot the kernel,
# and don't run a real test.  They're meant to demonstrate the basic
# functionality of kernel regression hunts and of using bkcvs for those
# hunts.

# 1.  Set up an rsync bkcvs repository; see tools/kernel_rsync.
#
# 2.  Update the reg_env file in this directory and source it.
#
        TOP=`pwd`
        . reg_env
#
# 3.  Set up some files used by the reghunt tools.
#
        bkcvs_setup
#
#     Take a look at the files CVS_ChangeSet.linux-2.5 and
#     CVS_Tag.linux-2.5 in the builds directory.
#
# 4.  Run a regression hunt whose config file is available in the
#     hunts/ directory.  The verbosity level for the tools is set to
#     let them write message about what they're doing, and reg_summary
#     can be run at any time to quickly see what's been tested and
#     how many times the build is failing.
#
        cd ${TOP}/hunts
        reg_search bug12345.config > bug12345.log 2>&1
        reg_summary < bug12345.log
#
# 5.  The previous hunt had several build failures.  The changeset IDs
#     for revisions with failing builds are recorded in
#     builds/CVS_Failures.linux-2.5.  You can see why each of them failed:
#
        cd ${TOP}/builds/logs.bug*
	grep '^make.* Error' make.*.log
#
#     and so on for each of them.  You'll notice that they all fail for
#     the same reason.  We can identify where the first failure occurs
#     and the patch which fixes it, and add all of the intervening IDs
#     to the list of known build failures.
#
# 6.  Identify the patch which introduced the build failure.  From the
#     output for bug12345, we can see that 12345 builds and 12376 does
#     not.
#
        cd ${TOP}/hunts
        reg_search failstart.config > failstart.log 2>&1
	reg_summary < failstart.log
#
# 7.  Identify the patch which fixed the build failure, based on the.      
#     last failure and first successsful build after it from the output
#     of the bug12345 search.
#
        reg_search failend.config > failend.log 2>&1
	reg_summary < failend.log
#
# 8.  Add the intervening range to the list of known failures:
#
        cd ${TOP}/builds
        reg_addfails 12366 12560 CVS_Failures.linux-2.5
#
# 9.  Search for the "cause" of bug 12346 , and notice that it doesn't
#     try to update or build revisions in the range of known build
#     failures.
#
        cd ${TOP}/hunts
        reg_search bug12346.config > bug12346.log 2>&1
        reg_summary < bug12346.log
#
# 10. Search for a bug which was introduced during the range for which
#     the kernel build fails.  We already determined that the build
#     succeeds starting with changeset 12561.  That's a rather large
#     changeset which includes several unrelated changes, but comments
#     let us guess that the changes to drivers/scsi/qla1280.[ch] are
#     what we want.
#
#     Identify which revisions of those files changed and make a patch:
#
        cd ${TOP}/builds/linux-2.5/drivers/scsi
	#cvs log -N qla1280.h | less
	#cvs log -N qla1280.c | less
	cd ../..
	cvs diff -up -r1.17 -r1.18 drivers/scsi/qla1280.h \
	   > ../qla1280.patch
	cvs diff -up -r1.35 -r1.36 drivers/scsi/qla1280.c \
	   >> ../qla1280.patch
#
#     Verify that the patch can be applied at the start of the failing
#     range and allows the build to succeed.
#
        bkcvs_update 12366
	cd ${TOP}/builds/linux-2.5
	patch -p0 < ../qla1280.patch
	kernel_build 12366
#
#     Perform the hunt using a special-purpose script for REG_UPDATE
#     which applies the patch for changesets within the failure range.
#
        cd ${TOP}/hunts
        reg_search bug12400.config > bug12400.log 2>&1
        reg_summary < bug12400.log


# 2003-12-11  Janis Johnson  <janis187@us.ibm.com>
